@@ -29,10 +29,6 @@ macro_rules! circuit_node_auto_leaf_impl {
     ($uuid:literal) => {
         circuit_node_auto_impl!($uuid, no_autoname);
 
-        fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-            Box::new([].into_iter())
-        }
-
         fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
             vec![]
         }
@@ -43,6 +39,10 @@ macro_rules! circuit_node_auto_leaf_impl {
         {
             Ok(self.clone())
         }
+
+        fn _replace_children(&self, _children: Vec<CircuitRc>) -> Result<Self> {
+            Ok(self.clone())
+        }
     };
 }
 
@@ -57,13 +57,8 @@ pub struct Array {
 circuit_node_extra_impl!(Array, self_hash_default);
 
 impl CircuitNodeComputeInfoImpl for Array {
-    fn compute_shape(&self) -> Shape {
-        self.value.shape().clone()
-    }
-    fn device_dtype_extra(&self) -> Box<dyn Iterator<Item = TorchDeviceDtypeOp> + '_> {
-        Box::new(std::iter::once(
-            TorchDeviceDtype::from_tensor(&self.value).into(),
-        ))
+    fn device_dtype_extra(&self) -> Option<Vec<TorchDeviceDtypeOp>> {
+        Some(vec![TorchDeviceDtype::from_tensor(&self.value).into()])
     }
 }
 
@@ -91,11 +86,10 @@ impl Array {
                 sym: SymbolicSizeProduct::from(value.shape()[i]).to_string(),
             })
         }
-        let mut out = Self {
+        let out = Self {
+            info: CachedCircuitInfo::incomplete(name, value.shape().clone(), vec![]),
             value,
-            info: Default::default(),
         };
-        out.info.name = name;
         out.initial_init_info()
     }
     pub fn from_hash_prefix(
@@ -213,9 +207,6 @@ pub struct Symbol {
 circuit_node_extra_impl!(Symbol, self_hash_default);
 
 impl CircuitNodeComputeInfoImpl for Symbol {
-    fn compute_shape(&self) -> Shape {
-        self.info.shape.clone() // note: assumes shape has already been initted!
-    }
     fn compute_flags(&self) -> CircuitFlags {
         self.compute_flags_default() & !CircuitFlags::IS_EXPLICITLY_COMPUTABLE
     }
@@ -244,12 +235,10 @@ impl CircuitNode for Symbol {
 impl Symbol {
     #[apply(new_rc)]
     pub fn new(shape: Shape, uuid: Uuid, name: Option<Name>) -> (Self) {
-        let mut out = Self {
+        let out = Self {
             uuid,
-            info: Default::default(),
+            info: CachedCircuitInfo::incomplete(name, shape, vec![]),
         };
-        out.info.shape = shape;
-        out.info.name = name;
         out.initial_init_info().unwrap()
     }
 }
@@ -287,11 +276,7 @@ pub struct Scalar {
 
 circuit_node_extra_impl!(Scalar, self_hash_default);
 
-impl CircuitNodeComputeInfoImpl for Scalar {
-    fn compute_shape(&self) -> Shape {
-        self.info().shape.clone() // note: assumes shape has already been initted!
-    }
-}
+impl CircuitNodeComputeInfoImpl for Scalar {}
 
 impl CircuitNodeHashItems for Scalar {
     fn compute_hash_non_name_non_children(&self, hasher: &mut blake3::Hasher) {
@@ -317,12 +302,10 @@ impl CircuitNode for Scalar {
 impl Scalar {
     #[apply(new_rc)]
     pub fn new(value: f64, shape: Shape, name: Option<Name>) -> (Self) {
-        let mut out = Self {
+        let out = Self {
             value,
-            info: Default::default(),
+            info: CachedCircuitInfo::incomplete(name, shape, vec![]),
         };
-        out.info.name = name;
-        out.info.shape = shape;
 
         out.initial_init_info().unwrap()
     }
