@@ -2020,6 +2020,7 @@ impl EqByBigHash for PyCircuitBase {
 }
 
 const DEFAULT_FANCY_VALIDATE: bool = false;
+const DEFAULT_ASSERT_ANY_FOUND: bool = false;
 
 #[pymethods]
 impl PyCircuitBase {
@@ -2344,7 +2345,8 @@ impl PyCircuitBase {
         transform,
         cache_transform = true,
         cache_update = true,
-        fancy_validate = DEFAULT_FANCY_VALIDATE
+        fancy_validate = DEFAULT_FANCY_VALIDATE,
+        assert_any_found = DEFAULT_ASSERT_ANY_FOUND
     ))]
     pub fn update(
         &self,
@@ -2353,6 +2355,7 @@ impl PyCircuitBase {
         cache_transform: bool,
         cache_update: bool,
         fancy_validate: bool,
+        assert_any_found: bool,
     ) -> PyResult<CircuitRc> {
         matcher.update(
             self.0.clone(),
@@ -2360,6 +2363,7 @@ impl PyCircuitBase {
             cache_transform,
             cache_update,
             fancy_validate,
+            assert_any_found,
         )
     }
     #[pyo3(signature=(matcher, fancy_validate = DEFAULT_FANCY_VALIDATE))]
