@@ -26,8 +26,6 @@ use crate::{
 #[pyclass(extends=PyCircuitBase)]
 #[derive(Clone)]
 pub struct Tag {
-    #[pyo3(get)]
-    pub node: CircuitRc,
     #[pyo3(get)]
     pub uuid: Uuid,
     info: CachedCircuitInfo,
@@ -36,12 +34,10 @@ pub struct Tag {
 impl Tag {
     #[apply(new_rc)]
     pub fn new(node: CircuitRc, uuid: Uuid, name: Option<Name>) -> (Self) {
-        let mut out = Self {
-            node,
+        let out = Self {
             uuid,
-            info: Default::default(),
+            info: CachedCircuitInfo::incomplete(name, node.shape().clone(), vec![node]),
         };
-        out.info.name = name;
         out.initial_init_info().unwrap()
     }
 }
@@ -50,18 +46,14 @@ impl CircuitNodeAutoName for Tag {
     const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
 
     fn auto_name(&self) -> Option<Name> {
-        child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone())
+        child_name_with_maybe_paren(&Self::PRIORITY, self.node().clone())
             .map(|n| (n.string() + " tag").into())
     }
 }
 
 circuit_node_extra_impl!(Tag, self_hash_default);
 
-impl CircuitNodeComputeInfoImpl for Tag {
-    fn compute_shape(&self) -> Shape {
-        self.node.shape().clone()
-    }
-}
+impl CircuitNodeComputeInfoImpl for Tag {}
 
 impl CircuitNodeHashItems for Tag {
     fn compute_hash_non_name_non_children(&self, hasher: &mut blake3::Hasher) {
@@ -73,23 +65,11 @@ impl CircuitNode for Tag {
     circuit_node_auto_impl!("63fdc4ce-2f1b-40b3-b8b6-13991b54cbd7");
 
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
-        vec![(0..self.node.info().rank()).map(Some).collect()]
+        vec![(0..self.node().info().rank()).map(Some).collect()]
     }
 
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new(std::iter::once(self.node.clone()))
-    }
-
-    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
-    where
-        Self: Sized,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
-        Ok(Self::new(
-            f(0, self.node.clone())?,
-            self.uuid,
-            self.info().name,
-        ))
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
+        Ok(Self::new(children[0].clone(), self.uuid, self.info().name))
     }
 
     fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
@@ -115,15 +95,16 @@ impl Tag {
     pub fn new_with_random_uuid(node: CircuitRc, name: Option<Name>) -> Self {
         Self::new(node, Uuid::new_v4(), name)
     }
+
+    #[getter]
+    pub fn node(&self) -> CircuitRc {
+        self.info.children[0].clone()
+    }
 }
 
 #[pyclass(extends=PyCircuitBase)]
 #[derive(Clone)]
 pub struct DiscreteVar {
-    #[pyo3(get)]
-    pub values: CircuitRc,
-    #[pyo3(get)]
-    pub probs_and_group: CircuitRc,
     info: CachedCircuitInfo,
 }
 
@@ -148,12 +129,14 @@ impl DiscreteVar {
                 probs: probs_and_group.info().shape[0],
             });
         }
-        let mut out = Self {
-            values,
-            probs_and_group,
-            info: Default::default(),
+
+        let out = Self {
+            info: CachedCircuitInfo::incomplete(
+                name,
+                values.info().shape[1..].iter().cloned().collect(),
+                vec![values, probs_and_group],
+            ),
         };
-        out.info.name = name;
         out.initial_init_info()
     }
 
@@ -176,7 +159,7 @@ impl CircuitNodeAutoName for DiscreteVar {
 
     fn auto_name(&self) -> Option<Name> {
         // Ignores probs and groups because we almost never care about the probs and groups name
-        child_name_with_maybe_paren(&Self::PRIORITY, self.values.clone())
+        child_name_with_maybe_paren(&Self::PRIORITY, self.values().clone())
             .map(|n| (n.string() + " var").into())
     }
 }
@@ -184,10 +167,6 @@ impl CircuitNodeAutoName for DiscreteVar {
 circuit_node_extra_impl!(DiscreteVar, self_hash_default);
 
 impl CircuitNodeComputeInfoImpl for DiscreteVar {
-    fn compute_shape(&self) -> Shape {
-        self.values.info().shape[1..].iter().cloned().collect()
-    }
-
     fn compute_flags(&self) -> CircuitFlags {
         (self.compute_flags_default() | CircuitFlags::CAN_BE_SAMPLED)
             & !CircuitFlags::IS_CONSTANT
@@ -209,20 +188,8 @@ impl CircuitNode for DiscreteVar {
         ]
     }
 
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new([self.values.clone(), self.probs_and_group.clone()].into_iter())
-    }
-
-    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
-    where
-        Self: Sized,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
-        Self::try_new(
-            f(0, self.values.clone())?,
-            f(1, self.probs_and_group.clone())?,
-            self.info().name,
-        )
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
+        Self::try_new(children[0].clone(), children[1].clone(), self.info().name)
     }
 
     fn eval_tensors(&self, _tensors: &[Tensor]) -> Result<Tensor> {
@@ -253,13 +220,20 @@ impl DiscreteVar {
         // TODO: naming
         Tag::new_with_random_uuid(Scalar::nrc(1.0 / (size as f64), sv![size], None), name)
     }
+    #[getter]
+    pub fn values(&self) -> &CircuitRc {
+        &self.info.children[0]
+    }
+    #[getter]
+    pub fn probs_and_group(&self) -> &CircuitRc {
+        &self.info.children[1]
+    }
 }
 
 #[pyclass(extends=PyCircuitBase)]
 #[derive(Clone)]
 pub struct StoredCumulantVar {
-    #[pyo3(get)]
-    pub cumulants: BTreeMap<usize, CircuitRc>, // using btreemap so it always iterates in order
+    pub cumulant_ixs: Vec<usize>,
     #[pyo3(get)]
     pub uuid: Uuid,
     info: CachedCircuitInfo,
@@ -294,12 +268,15 @@ impl StoredCumulantVar {
                 },);
             }
         }
-        let mut out = Self {
-            cumulants,
+        let out = Self {
+            cumulant_ixs: cumulants.iter().map(|(a, _)| *a).collect(),
             uuid,
-            info: Default::default(),
+            info: CachedCircuitInfo::incomplete(
+                name,
+                cumulants[&1].info().shape.clone(),
+                cumulants.iter().map(|(_, b)| b).cloned().collect(),
+            ),
         };
-        out.info.name = name;
         out.initial_init_info()
     }
 
@@ -323,13 +300,13 @@ impl StoredCumulantVar {
 
 impl CircuitNodeAutoName for StoredCumulantVar {
     fn auto_name(&self) -> Option<Name> {
-        if self.cumulants.values().any(|x| x.info().name.is_none()) {
+        if self.children().any(|x| x.info().name.is_none()) {
             None
         } else {
             Some(
                 ("StoredCumulantVar(".to_owned()
                     + &self
-                        .cumulants
+                        .cumulants()
                         .iter()
                         .map(|(r, x)| {
                             format!(
@@ -349,15 +326,11 @@ impl CircuitNodeAutoName for StoredCumulantVar {
 circuit_node_extra_impl!(StoredCumulantVar, self_hash_default);
 
 impl CircuitNodeComputeInfoImpl for StoredCumulantVar {
-    fn compute_shape(&self) -> Shape {
-        self.cumulants[&1].info().shape.clone()
-    }
-
     fn compute_flags(&self) -> CircuitFlags {
         let flags = self.compute_flags_default()
             & !CircuitFlags::IS_CONSTANT
             & !CircuitFlags::IS_EXPLICITLY_COMPUTABLE;
-        if self.cumulants.keys().max().unwrap() <= &2 {
+        if self.cumulant_ixs.iter().max().unwrap() <= &2 {
             flags | CircuitFlags::CAN_BE_SAMPLED
         } else {
             flags & !CircuitFlags::CAN_BE_SAMPLED
@@ -367,7 +340,7 @@ impl CircuitNodeComputeInfoImpl for StoredCumulantVar {
 
 impl CircuitNodeHashItems for StoredCumulantVar {
     fn compute_hash_non_name_non_children(&self, hasher: &mut blake3::Hasher) {
-        for k in self.cumulants.keys() {
+        for k in &self.cumulant_ixs {
             hasher.update(&k.to_le_bytes());
         }
         hasher.update(self.uuid.as_bytes());
@@ -378,27 +351,14 @@ impl CircuitNode for StoredCumulantVar {
     circuit_node_auto_impl!("f36da959-d160-484d-b6b8-7685ef7521c0");
 
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
-        self.cumulants
-            .values()
+        self.children()
             .map(|x| vec![None; x.info().rank()])
             .collect()
     }
 
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new(self.cumulants.values().cloned())
-    }
-
-    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
-    where
-        Self: Sized,
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
         Self::try_new(
-            self.cumulants
-                .iter()
-                .enumerate()
-                .map(|(i, (k, v))| f(i, v.clone()).map(|z| (*k, z)))
-                .collect::<Result<BTreeMap<_, _>, _>>()?,
+            self.cumulant_ixs.iter().cloned().zip(children).collect(),
             self.uuid,
             self.info().name,
         )
@@ -436,4 +396,13 @@ impl StoredCumulantVar {
     ) -> Result<Self> {
         Self::new_mv(mean, variance, higher_cumulants, uuid, name)
     }
+
+    #[getter]
+    fn cumulants(&self) -> BTreeMap<usize, CircuitRc> {
+        self.cumulant_ixs
+            .iter()
+            .cloned()
+            .zip(self.children())
+            .collect()
+    }
 }
