@@ -484,7 +484,8 @@ pub fn compiler_simp_py(circuit: CircuitRc) -> CircuitRc {
 }
 
 pub fn compiler_simp(circ: CircuitRc, opt_context: &mut OptimizationContext) -> CircuitRc {
-    SimpFnSubset::compiler_default().simp_ctx(circ, opt_context)
+    let set = opt_context.settings.simp_fn_subset.clone(); // appease borrow checker
+    set.simp_ctx(circ, opt_context)
 }
 
 #[pyfunction]
@@ -617,7 +618,7 @@ impl CircBloomFilter {
 #[inline]
 fn child_bloom_filter(add: &Add) -> CircBloomFilter {
     let mut result: CircBloomFilter = Default::default();
-    for node in &add.nodes {
+    for node in add.children() {
         result.insert(node.info().hash_usize() as u64);
     }
     result
@@ -671,11 +672,11 @@ pub fn deep_heuristic_nest_adds(circ: CircuitRc) -> CircuitRc {
         let timeout_timer = std::time::Instant::now();
         let mut to_do: Vec<((Add, CircBloomFilter), (Add, CircBloomFilter))> = vec![];
         for cand_sup in &seen_adds {
-            if cand_sup.0.nodes.len() >= 3 {
+            if cand_sup.0.num_children() >= 3 {
                 for cand_sub in &seen_adds {
                     // stats.0 += 1;
                     if cand_sup.1.is_superset(&cand_sub.1)
-                        && cand_sub.0.nodes.len() < cand_sup.0.nodes.len()
+                        && cand_sub.0.num_children() < cand_sup.0.num_children()
                         && cand_sub.0 != cand_sup.0
                     {
                         // stats.1 += 1;
@@ -731,7 +732,7 @@ pub fn deep_heuristic_nest_adds(circ: CircuitRc) -> CircuitRc {
 }
 
 pub fn add_nest_ltr(add: &Add) -> Add {
-    let (l, r) = add.nodes.split_at(2.min(add.nodes.len()));
+    let (l, r) = add.children_sl().split_at(2.min(add.num_children()));
     let base = Add::new(l.to_vec(), None);
     r.iter()
         .fold(base, |acc, x| Add::new(vec![acc.rc(), x.clone()], None))
