@@ -10,7 +10,7 @@ where
             .map(|x| should_sample(x))
             .unwrap_or(false)
         {
-            Some(c.as_discrete_var().unwrap().values.clone())
+            Some(c.as_discrete_var().unwrap().values().clone())
         } else {
             None
         }
