@@ -33,8 +33,8 @@ use smallvec::SmallVec as Sv;
 use thiserror::Error;
 
 use crate::{
-    iterative_matcher::function_per_child, Expander, IterateMatchResults, IterativeMatcherRc,
-    Matcher, Transform, TransformRc,
+    iterative_matcher::function_per_child, restrict, Expander, IterateMatchResults,
+    IterativeMatcherRc, Matcher, Transform, TransformRc,
 };
 
 fn_struct!(pub Seeder:Fn(circuit:CircuitRc)->i64);
@@ -53,14 +53,14 @@ fn count_elements(s: &Shape) -> usize {
 
 #[pyfunction]
 pub fn default_var_matcher() -> IterativeMatcherRc {
-    Matcher::types(vec![CircuitType::DiscreteVar])
-        .filter(
-            true,
-            None,
-            None,
-            Matcher::types(vec![CircuitType::Cumulant]).into(),
-        )
-        .into()
+    restrict(
+        Matcher::types(vec![CircuitType::DiscreteVar]).into(),
+        true,
+        None,
+        None,
+        Matcher::types(vec![CircuitType::Cumulant]).into(),
+    )
+    .into()
 }
 
 pub trait SampleSpecTrait {
@@ -85,6 +85,7 @@ pub trait SampleSpecTrait {
         self,
         var_matcher: IterativeMatcherRc,
         default_fancy_validate: bool,
+        default_assert_any_found: bool,
         suffix: Option<String>,
     ) -> Expander
     where
@@ -94,6 +95,7 @@ pub trait SampleSpecTrait {
             vec![(var_matcher, self.get_transform())],
             true,
             default_fancy_validate,
+            default_assert_any_found,
             suffix,
         )
     }
@@ -142,15 +144,20 @@ macro_rules! py_forward_sample_spec {
             fn get_transform_py(&self) -> TransformRc {
                 self.clone().get_transform()
             }
-            #[pyo3(name = "get_sample_expander", signature=(var_matcher = default_var_matcher(), default_fancy_validate = false, suffix = "sample".to_owned()))]
+            #[pyo3(name = "get_sample_expander", signature=(var_matcher = default_var_matcher(), default_fancy_validate = false, default_assert_any_found = false, suffix = "sample".to_owned()))]
             fn get_sample_expander_py(
                 &self,
                 var_matcher: IterativeMatcherRc,
                 default_fancy_validate: bool,
+                default_assert_any_found: bool,
                 suffix: Option<String>,
             ) -> Expander {
-                self.clone()
-                    .get_sample_expander(var_matcher, default_fancy_validate, suffix)
+                self.clone().get_sample_expander(
+                    var_matcher,
+                    default_fancy_validate,
+                    default_assert_any_found,
+                    suffix,
+                )
             }
         }
 
@@ -536,7 +543,7 @@ impl Sampler {
     ) -> Self {
         let expander = sample_spec
             .clone()
-            .get_sample_expander(var_matcher, false, suffix);
+            .get_sample_expander(var_matcher, false, false, suffix);
         Self {
             expander,
             cumulant_matcher,
@@ -554,7 +561,7 @@ impl Sampler {
         self.estimate(circuit)
     }
     pub fn sample(&mut self, circuit: CircuitRc) -> Result<CircuitRc> {
-        let out = self.expander.batch(circuit.clone(), None)?;
+        let out = self.expander.batch(circuit.clone(), None, None)?;
         let is_orig_shape = out.shape() == circuit.shape();
         let is_sampled_shape = out.shape()
             == &self
