@@ -1,21 +1,19 @@
 use anyhow::{bail, Result};
 use circuit_base::{prelude::*, Einsum};
 use circuit_rewrites::algebraic_rewrite::{distribute_once_raw, DistributeError};
-use get_update_node::{IterativeMatcher, IterativeMatcherRc, MatcherData};
+use get_update_node::{new_traversal, IterativeMatcherRc, MatcherData};
 use macro_rules_attribute::apply;
 use pyo3::prelude::*;
 use rr_util::{cached_lambda, util::HashBytes};
 
 #[pyfunction]
 pub fn traverse_until_depth(depth: Option<u32>) -> IterativeMatcherRc {
-    IterativeMatcher::noop_traversal()
-        .filter(
-            false,
-            None,
-            depth.map(|i| i + 1),
-            MatcherData::Always(false).into(),
-        )
-        .rc()
+    new_traversal(
+        None,
+        depth.map(|i| i + 1),
+        MatcherData::Always(false).into(),
+    )
+    .rc()
 }
 
 // TODO: improve me: fixed operand index isn't very flexible in deep case (though I'm not sure that deep distribute rewrite is that important...)
