@@ -1,21 +1,19 @@
 use anyhow::Result;
 use circuit_base::{prelude::*, CircuitType, Index};
 use circuit_rewrites::algebraic_rewrite::{index_elim_identity, push_down_index_raw};
-use get_update_node::{IterativeMatcher, IterativeMatcherRc, Matcher};
+use get_update_node::{new_traversal, IterativeMatcherRc, Matcher};
 use macro_rules_attribute::apply;
 use pyo3::prelude::*;
 use rr_util::{cached_lambda, util::HashBytes};
 
 #[pyfunction]
 pub fn default_index_traversal() -> IterativeMatcherRc {
-    IterativeMatcher::noop_traversal()
-        .filter(
-            false,
-            None,
-            None,
-            Matcher::types(vec![CircuitType::Index, CircuitType::Array]).rc(),
-        )
-        .rc()
+    new_traversal(
+        None,
+        None,
+        Matcher::types(vec![CircuitType::Index, CircuitType::Array]).rc(),
+    )
+    .rc()
 }
 
 #[pyfunction]
