@@ -498,12 +498,11 @@ impl RearrangeSpec {
         self.to_string_impl(false, false, false)
     }
 
-    #[pyo3(name = "to_string")]
-    #[args(
-        size_on_letter = "true",
-        use_symbolic = "true",
-        show_size_if_present = "true"
-    )]
+    #[pyo3(name = "to_string", signature = (
+        size_on_letter = true,
+        use_symbolic = true,
+        show_size_if_present = true
+    ))]
     pub fn to_string_impl(
         &self,
         size_on_letter: bool,
@@ -753,7 +752,7 @@ impl RearrangeSpec {
     /// .unwrap();
     /// assert_eq!(spec.canonicalize(true), canonicalized_spec);
     /// ```
-    #[args(special_case_ones = true)]
+    #[pyo3(signature=(special_case_ones = true))]
     pub fn canonicalize(&self, special_case_ones: bool) -> Self {
         // Remove all indices of size 1
         let int_sizes: Vec<Option<usize>> = self.int_sizes.iter().map(|x| (*x).into()).collect();
