@@ -145,10 +145,6 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_class::<get_update_node::Matcher>()?;
     m.add_class::<get_update_node::matcher::RegexWrap>()?;
     m.add_class::<get_update_node::IterativeMatcher>()?;
-    m.add(
-        "traversal",
-        get_update_node::IterativeMatcher::noop_traversal(),
-    )?;
     m.add_function(wrap_pyfunction!(get_update_node::restrict, m)?)?;
     m.add_function(wrap_pyfunction!(get_update_node::restrict_sl, m)?)?;
     m.add_function(wrap_pyfunction!(get_update_node::new_traversal, m)?)?;
